/*   **********************************************************************  **
 **   Copyright notice                                                       **
 **                                                                          **
 **   (c) 2003-2005 RSSOwl Development Team                                  **
 **   http://www.rssowl.org/                                                 **
 **                                                                          **
 **   All rights reserved                                                    **
 **                                                                          **
 **   This program and the accompanying materials are made available under   **
 **   the terms of the Common Public License v1.0 which accompanies this     **
 **   distribution, and is available at:                                     **
 **   http://www.rssowl.org/legal/cpl-v10.html                               **
 **                                                                          **
 **   A copy is found in the file cpl-v10.html and important notices to the  **
 **   license from the team is found in the textfile LICENSE.txt distributed **
 **   in this package.                                                       **
 **                                                                          **
 **   This copyright notice MUST APPEAR in all copies of the file!           **
 **                                                                          **
 **   Contributors:                                                          **
 **     RSSOwl - initial API and implementation (bpasero@rssowl.org)         **
 **                                                                          **
 **  **********************************************************************  */

package net.sourceforge.rssowl.controller;

import net.sourceforge.rssowl.controller.dialog.SearchDialog;
import net.sourceforge.rssowl.controller.thread.ExtendedThread;
import net.sourceforge.rssowl.model.Category;
import net.sourceforge.rssowl.util.GlobalSettings;
import net.sourceforge.rssowl.util.i18n.ITranslatable;
import net.sourceforge.rssowl.util.shop.FontShop;
import net.sourceforge.rssowl.util.shop.HotkeyShop;
import net.sourceforge.rssowl.util.shop.LayoutShop;
import net.sourceforge.rssowl.util.shop.PaintShop;
import net.sourceforge.rssowl.util.shop.RegExShop;
import net.sourceforge.rssowl.util.shop.StringShop;
import net.sourceforge.rssowl.util.shop.WidgetShop;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DropTarget;
import org.eclipse.swt.dnd.DropTargetAdapter;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.eclipse.swt.dnd.TextTransfer;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.ToolBar;
import org.eclipse.swt.widgets.ToolItem;

import java.util.Vector;

/**
 * This is the RSS Quickview controll that is displayed directly under the
 * menue.
 * 
 * @author <a href="mailto:bpasero@rssowl.org">Benjamin Pasero </a>
 * @version 1.1.3
 */
public class Quickview implements ITranslatable, IFontChangeable {

  /** Delay before the Search is beginning */
  private static final int SEARCH_DELAY = 400;

  private MenuItem aggregateFavorites;
  private Composite bottomContainer;
  private ToolItem discoverFeedItem;
  private MenuItem exportOPML;
  private MenuItem importBlogroll;
  private MenuItem importOPML;
  private CLabel liveSearchLabel;
  private MenuItem markAllRead;
  private ToolItem markReadItem;
  private MenuItem newCategory;
  private MenuItem newFavorite;
  private MenuItem newSubCategory;
  private ToolItem nextUnreadNewsItem;
  private Menu quickviewMenu;
  private MenuItem reloadFavorites;
  private ToolItem reloadItem;
  private ExtendedThread searchDelayThread;
  private MenuItem searchFavorites;
  private ToolItem searchFeedItem;
  private ToolItem searchItem;
  private ViewForm toolbarViewForm;
  private Composite topContainer;
  private Label urlLabelQuickView;
  private ToolItem validateFeedItem;
  DropTarget comboDNDTarget;
  EventManager eventManager;
  ToolItem exportItem;
  Menu exportMenu;
  ToolItem favoritesItem;
  Menu favoritesMenu;
  ToolItem importItem;
  Menu importMenu;
  Text liveSearch;
  ToolItem newItem;
  Menu newMenu;
  MenuItem quickviewState;
  GUI rssOwlGui;
  DropTarget searchDNDTarget;
  Shell shell;
  boolean stopLiveSearch;
  MenuItem toolBarState;
  ToolBar topToolBar;
  Combo urlTestCombo;

  /**
   * Instantiate a new Quickview
   * 
   * @param rssOwlGui Access some methods of the main controller
   * @param shell The shell
   * @param eventManager The event manager
   */
  public Quickview(GUI rssOwlGui, Shell shell, EventManager eventManager) {
    this.rssOwlGui = rssOwlGui;
    this.shell = shell;
    this.eventManager = eventManager;
    initComponents();
  }

  /**
   * Clear the live search input field
   */
  public void clearLiveSearch() {
    stopLiveSearch = true;
    liveSearch.setText("");
    stopLiveSearch = false;
  }

  /**
   * Disable the "New" dropdown ToolItem
   */
  public void setNewMenuDisabled() {
    if (WidgetShop.isset(newItem))
      newItem.setEnabled(false);
  }

  /**
   * Show / Hide the quickview control
   * 
   * @param show TRUE if visible
   * @param save If TRUE save state to GlobalSettings
   */
  public void setShowQuickview(boolean show, boolean save) {

    /** Update */
    rssOwlGui.getRSSOwlMenu().setQuickviewMenuEnabled(show);

    /** Set ViewForm Content */
    toolbarViewForm.setContent((show == true) ? bottomContainer : null, true);

    /** Update visibility */
    bottomContainer.setVisible(show);

    /** Completly hide the ViewForm since it contains no controls now */
    if (!show && !topContainer.getVisible())
      setShowViewForm(false);

    /** Restore the ViewForm since it is now containing a control */
    else if (show && !topContainer.getVisible())
      setShowViewForm(true);

    /** This might be called from a closing RSSOwl, so be carefull */
    if (WidgetShop.isset(shell) && GUI.isAlive())
      shell.layout();

    /** Remove Focus from the Quickview Control */
    if (!show)
      shell.setFocus();

    /** Update Global Settings */
    if (save)
      GlobalSettings.isQuickviewShown = show;

    /** Update Context Menu */
    quickviewState.setSelection(show);
  }

  /**
   * Show / Hide the ToolBar control
   * 
   * @param show TRUE if visible
   * @param save If TRUE save state to GlobalSettings
   */
  public void setShowToolBar(boolean show, boolean save) {

    /** Update */
    rssOwlGui.getRSSOwlMenu().setToolBarMenuEnabled(show);

    /** Set ViewForm TopLeft */
    toolbarViewForm.setTopLeft((show == true) ? topContainer : null, true);

    /** Update visibility */
    topContainer.setVisible(show);

    /** Completly hide the ViewForm since it contains no controls now */
    if (!show && !bottomContainer.getVisible())
      setShowViewForm(false);

    /** Restore the ViewForm since it is now containing a control */
    else if (show && !bottomContainer.getVisible())
      setShowViewForm(true);

    /** This might be called from a closing RSSOwl, so be carefull */
    if (WidgetShop.isset(shell) && GUI.isAlive())
      shell.layout();

    /** Update Global Settings */
    if (save)
      GlobalSettings.isToolBarShown = show;

    /** Update Context Menu */
    toolBarState.setSelection(show);
  }

  /**
   * Show / Hide the ViewForm containing ToolBar and Quickview
   * 
   * @param show TRUE if visible
   */
  public void setShowViewForm(boolean show) {

    /** Set Visibility */
    toolbarViewForm.setVisible(show);

    /** Set state of Exclude in GridData */
    ((GridData) toolbarViewForm.getLayoutData()).exclude = !show;
  }

  /**
   * @see net.sourceforge.rssowl.controller.IFontChangeable#updateFonts()
   */
  public void updateFonts() {
    urlLabelQuickView.setFont(FontShop.dialogFont);
    urlTestCombo.setFont(FontShop.dialogFont);
    liveSearch.setFont(FontShop.dialogFont);
  }

  /** Update all controlls text with i18n */
  public void updateI18N() {
    urlLabelQuickView.setText(GUI.i18n.getTranslation("LABEL_ADDRESS"));
    liveSearchLabel.setToolTipText(GUI.i18n.getTranslation("TOOLTIP_QUICKSEARCH"));
    toolBarState.setText(GUI.i18n.getTranslation("MENU_TOOLBAR"));
    quickviewState.setText(GUI.i18n.getTranslation("MENU_QUICKVIEW"));

    /** Top ToolBar */
    newItem.setToolTipText(GUI.i18n.getTranslation("POP_NEW"));

    /** Set Text and Accelerator */
    if (WidgetShop.isset(newFavorite) && HotkeyShop.isHotkeySet("MENU_NEW_FAVORITE") && !GlobalSettings.isMac())
      newFavorite.setText(GUI.i18n.getTranslation("LABEL_FAVORITE") + "..." + "\t" + HotkeyShop.getHotkeyName("MENU_NEW_FAVORITE"));
    else if (WidgetShop.isset(newFavorite))
      newFavorite.setText(GUI.i18n.getTranslation("LABEL_FAVORITE") + "...");

    if (WidgetShop.isset(newCategory))
      newCategory.setText(GUI.i18n.getTranslation("LABEL_CATEGORY") + "...");

    if (WidgetShop.isset(newSubCategory))
      newSubCategory.setText(GUI.i18n.getTranslation("POP_SUB_CATEGORY") + "...");

    importItem.setToolTipText(GUI.i18n.getTranslation("POP_IMPORT"));
    importOPML.setText(GUI.i18n.getTranslation("POP_FROM_OPML") + "...");
    importBlogroll.setText(GUI.i18n.getTranslation("POP_IMPORT_BLOGROLL") + "...");
    exportItem.setToolTipText(GUI.i18n.getTranslation("BUTTON_EXPORT"));
    exportOPML.setText(GUI.i18n.getTranslation("POP_EXPORT_OPML") + "...");
    reloadItem.setToolTipText(GUI.i18n.getTranslation("MENU_RELOAD"));
    searchItem.setToolTipText(GUI.i18n.getTranslation("BUTTON_SEARCH") + "...");
    markReadItem.setToolTipText(GUI.i18n.getTranslation("POP_MARK_ALL_READ"));
    favoritesItem.setToolTipText(GUI.i18n.getTranslation("HEADER_RSS_FAVORITES"));

    /** Set Text and Accelerator */
    if (HotkeyShop.isHotkeySet("BUTTON_MARK_ALL_READ") && !GlobalSettings.isMac())
      markAllRead.setText(GUI.i18n.getTranslation("BUTTON_MARK_ALL_READ") + "\t" + HotkeyShop.getHotkeyName("BUTTON_MARK_ALL_READ"));
    else
      markAllRead.setText(GUI.i18n.getTranslation("BUTTON_MARK_ALL_READ"));

    /** Set Text and Accelerator */
    if (HotkeyShop.isHotkeySet("BUTTON_AGGREGATE_ALL") && !GlobalSettings.isMac())
      aggregateFavorites.setText(GUI.i18n.getTranslation("BUTTON_AGGREGATE_ALL") + "\t" + HotkeyShop.getHotkeyName("BUTTON_AGGREGATE_ALL"));
    else
      aggregateFavorites.setText(GUI.i18n.getTranslation("BUTTON_AGGREGATE_ALL"));

    /** Set Text and Accelerator */
    if (HotkeyShop.isHotkeySet("BUTTON_RELOAD_ALL") && !GlobalSettings.isMac())
      reloadFavorites.setText(GUI.i18n.getTranslation("BUTTON_RELOAD_ALL") + "\t" + HotkeyShop.getHotkeyName("BUTTON_RELOAD_ALL"));
    else
      reloadFavorites.setText(GUI.i18n.getTranslation("BUTTON_RELOAD_ALL"));

    /** Set Text and Accelerator */
    if (HotkeyShop.isHotkeySet("BUTTON_SEARCH_ALL") && !GlobalSettings.isMac())
      searchFavorites.setText(GUI.i18n.getTranslation("BUTTON_SEARCH_ALL") + "..." + "\t" + HotkeyShop.getHotkeyName("BUTTON_SEARCH_ALL"));
    else
      searchFavorites.setText(GUI.i18n.getTranslation("BUTTON_SEARCH_ALL") + "...");

    nextUnreadNewsItem.setToolTipText(GUI.i18n.getTranslation("MENU_NEXT_UNREAD_NEWS"));
    searchFeedItem.setToolTipText(GUI.i18n.getTranslation("MENU_FEEDSEARCH") + "...");
    discoverFeedItem.setToolTipText(GUI.i18n.getTranslation("MENU_FEED_DISCOVERY") + "...");
    validateFeedItem.setToolTipText(GUI.i18n.getTranslation("MENU_VALIDATE") + "...");
  }

  /**
   * Update the new menu in dependance of the selection in the favorites tree.
   * 
   * @param showSubCategory If TRUE, show the "New > Sub Category" item.
   * @param isBlogroll If TRUE, selection is set on a Blogroll.
   */
  public void updateNewMenu(final boolean showSubCategory, final boolean isBlogroll) {

    /** Re-Enable if required */
    if (!newItem.isEnabled())
      newItem.setEnabled(true);

    /** Dispose the old menu if set */
    if (WidgetShop.isset(newMenu))
      newMenu.dispose();

    /** Create New Menu */
    newMenu = new Menu(topToolBar);

    /** "New Favorite" is not allowed on a Blogroll */
    if (!isBlogroll) {

      /** New Favorite Item */
      newFavorite = new MenuItem(newMenu, SWT.PUSH);
      newFavorite.setImage(PaintShop.loadImage("/img/icons/add_feed.gif"));
      newFavorite.addDisposeListener(DisposeListenerImpl.getInstance());

      /** Set Text and Accelerator */
      if (HotkeyShop.isHotkeySet("MENU_NEW_FAVORITE") && !GlobalSettings.isMac())
        newFavorite.setText(GUI.i18n.getTranslation("LABEL_FAVORITE") + "..." + "\t" + HotkeyShop.getHotkeyName("MENU_NEW_FAVORITE"));
      else
        newFavorite.setText(GUI.i18n.getTranslation("LABEL_FAVORITE") + "...");

      newFavorite.addSelectionListener(new SelectionAdapter() {
        public void widgetSelected(SelectionEvent e) {
          eventManager.actionNewFavorite();
        }
      });

      /** Separator */
      new MenuItem(newMenu, SWT.SEPARATOR);
    }

    /** New Category Item */
    newCategory = new MenuItem(newMenu, SWT.PUSH);
    newCategory.setImage(PaintShop.loadImage("/img/icons/add_category_pop.gif"));
    newCategory.addDisposeListener(DisposeListenerImpl.getInstance());
    newCategory.setText(GUI.i18n.getTranslation("LABEL_CATEGORY") + "...");
    newCategory.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent e) {
        eventManager.actionNewCategory(!showSubCategory);
      }
    });

    /** "New Sub-Category" only allowed if selection is on a category */
    if (showSubCategory) {

      /** New Sub-Category Item */
      newSubCategory = new MenuItem(newMenu, SWT.PUSH);
      newSubCategory.setImage(PaintShop.loadImage("/img/icons/subcat.gif"));
      newSubCategory.addDisposeListener(DisposeListenerImpl.getInstance());
      newSubCategory.setText(GUI.i18n.getTranslation("POP_SUB_CATEGORY") + "...");
      newSubCategory.addSelectionListener(new SelectionAdapter() {
        public void widgetSelected(SelectionEvent e) {
          eventManager.actionNewCategory(true);
        }
      });
    }
  }

  /**
   * Create the Top ToolBar
   * 
   * @param topToolBarContainer The Top ToolBar Container
   */
  private void createToolBar(Composite topToolBarContainer) {

    /** Top ToolBar */
    topToolBar = new ToolBar(topToolBarContainer, SWT.FLAT | SWT.NO_FOCUS);
    topToolBar.setBackground(PaintShop.grayViewFormColor);
    topToolBar.setMenu(quickviewMenu);

    /** Drop Down: New Favorite and New Category */
    newItem = new ToolItem(topToolBar, SWT.DROP_DOWN);
    newItem.setImage(PaintShop.loadImage("/img/icons/add_category_small.gif"));
    newItem.setDisabledImage(PaintShop.loadImage("/img/icons/new_disabled.gif"));
    newItem.addDisposeListener(DisposeListenerImpl.getInstance());

    /** Handle Drop Down Selection */
    newItem.addListener(SWT.Selection, new Listener() {
      public void handleEvent(Event event) {

        /** Make popupmenue visible */
        if (event.detail == SWT.ARROW) {
          Rectangle rect = newItem.getBounds();
          Point pt = new Point(rect.x, rect.y + rect.height);
          pt = topToolBar.toDisplay(pt);
          newMenu.setLocation(pt.x, pt.y);
          newMenu.setVisible(true);
        }

        /** Perform default operation: New Favorite */
        else {
          eventManager.actionNewFavorite();
        }
      }
    });

    /** Create the New Menu */
    updateNewMenu(false, false);

    /** Drop Down: Import OPML, Import Synchronized Blogroll */
    importItem = new ToolItem(topToolBar, SWT.DROP_DOWN);
    importItem.setImage(PaintShop.loadImage("/img/icons/import.gif"));
    importItem.addDisposeListener(DisposeListenerImpl.getInstance());

    /** Import Menu */
    importMenu = new Menu(topToolBar);

    /** Handle Drop Down Selection */
    importItem.addListener(SWT.Selection, new Listener() {
      public void handleEvent(Event event) {

        /** Make popupmenue visible */
        if (event.detail == SWT.ARROW) {
          Rectangle rect = importItem.getBounds();
          Point pt = new Point(rect.x, rect.y + rect.height);
          pt = topToolBar.toDisplay(pt);
          importMenu.setLocation(pt.x, pt.y);
          importMenu.setVisible(true);
        }

        /** Perform default operation */
        else {
          eventManager.actionImportOPML();
        }
      }
    });

    /** Import OPML */
    importOPML = new MenuItem(importMenu, SWT.PUSH);
    importOPML.setImage(PaintShop.loadImage("/img/icons/import_opml.gif"));
    importOPML.addDisposeListener(DisposeListenerImpl.getInstance());
    importOPML.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent e) {
        eventManager.actionImportOPML();
      }
    });

    /** Import Synchronized Blogroll */
    importBlogroll = new MenuItem(importMenu, SWT.PUSH);
    importBlogroll.setImage(PaintShop.loadImage("/img/icons/add_subscription.gif"));
    importBlogroll.addDisposeListener(DisposeListenerImpl.getInstance());
    importBlogroll.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent e) {
        eventManager.actionImportBlogroll();
      }
    });

    /** Drop Down: Export To OPML */
    exportItem = new ToolItem(topToolBar, SWT.DROP_DOWN);
    exportItem.setImage(PaintShop.loadImage("/img/icons/export.gif"));
    exportItem.addDisposeListener(DisposeListenerImpl.getInstance());

    /** Export Menu */
    exportMenu = new Menu(topToolBar);

    /** Handle Drop Down Selection */
    exportItem.addListener(SWT.Selection, new Listener() {
      public void handleEvent(Event event) {

        /** Make popupmenue visible */
        if (event.detail == SWT.ARROW) {
          Rectangle rect = exportItem.getBounds();
          Point pt = new Point(rect.x, rect.y + rect.height);
          pt = topToolBar.toDisplay(pt);
          exportMenu.setLocation(pt.x, pt.y);
          exportMenu.setVisible(true);
        }

        /** Perform default operation */
        else {
          eventManager.actionCategoryToOPML(Category.getRootCategory(), "rssowl");
        }
      }
    });

    /** Export To OPML */
    exportOPML = new MenuItem(exportMenu, SWT.PUSH);
    exportOPML.setImage(PaintShop.loadImage("/img/icons/export_opml.gif"));
    exportOPML.addDisposeListener(DisposeListenerImpl.getInstance());
    exportOPML.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent e) {
        eventManager.actionCategoryToOPML(Category.getRootCategory(), "rssowl");
      }
    });

    /** Separator */
    new ToolItem(topToolBar, SWT.SEPARATOR);

    /** Reload Feed */
    reloadItem = new ToolItem(topToolBar, SWT.PUSH);
    reloadItem.setImage(PaintShop.iconReload);
    reloadItem.setDisabledImage(PaintShop.loadImage("/img/icons/reload_disabled.gif"));
    reloadItem.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent e) {
        eventManager.actionReload();
      }
    });

    /** Register to MenuManager */
    MenuManager.registerReloadToolItem(reloadItem);

    /** Search Feed */
    searchItem = new ToolItem(topToolBar, SWT.PUSH);
    searchItem.setImage(PaintShop.iconFind);
    searchItem.setDisabledImage(PaintShop.loadImage("/img/icons/search_disabled.gif"));
    searchItem.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent e) {
        eventManager.actionSearch();
      }
    });

    /** Register to MenuManager */
    MenuManager.registerSearchToolItem(searchItem);

    /** Mark Feed Read */
    markReadItem = new ToolItem(topToolBar, SWT.PUSH);
    markReadItem.setImage(PaintShop.loadImage("/img/icons/mark_feed_read.gif"));
    markReadItem.setDisabledImage(PaintShop.loadImage("/img/icons/mark_feed_read_disabled.gif"));
    markReadItem.addDisposeListener(DisposeListenerImpl.getInstance());
    markReadItem.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent e) {
        eventManager.actionMarkAllNewsRead();
      }
    });

    /** Register to MenuManager */
    MenuManager.registerMarkAllReadToolItem(markReadItem);

    /** Separator */
    new ToolItem(topToolBar, SWT.SEPARATOR);

    /** Next Unread News */
    nextUnreadNewsItem = new ToolItem(topToolBar, SWT.PUSH);
    nextUnreadNewsItem.setImage(PaintShop.loadImage("/img/icons/next_unread.gif"));
    nextUnreadNewsItem.setDisabledImage(PaintShop.loadImage("/img/icons/next_unread_disabled.gif"));
    nextUnreadNewsItem.addDisposeListener(DisposeListenerImpl.getInstance());
    nextUnreadNewsItem.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent e) {
        eventManager.actionGotoNextUnreadNews();
      }
    });

    /** Register to MenuManager */
    MenuManager.registerGotoNextUnreadNews(nextUnreadNewsItem);

    /** Separator */
    new ToolItem(topToolBar, SWT.SEPARATOR);

    /** Drop Down: Mark All Read, Aggregate Favs, Reload Favs, Search Favs */
    favoritesItem = new ToolItem(topToolBar, SWT.DROP_DOWN);
    favoritesItem.setImage(PaintShop.loadImage("/img/icons/favorites.gif"));
    favoritesItem.addDisposeListener(DisposeListenerImpl.getInstance());

    /** Favorites Menu */
    favoritesMenu = new Menu(topToolBar);

    /** Handle Drop Down Selection */
    favoritesItem.addListener(SWT.Selection, new Listener() {
      public void handleEvent(Event event) {

        /** Make popupmenue visible */
        Rectangle rect = favoritesItem.getBounds();
        Point pt = new Point(rect.x, rect.y + rect.height);
        pt = topToolBar.toDisplay(pt);
        favoritesMenu.setLocation(pt.x, pt.y);
        favoritesMenu.setVisible(true);
      }
    });

    /** Aggregate all Favorites */
    aggregateFavorites = new MenuItem(favoritesMenu, SWT.PUSH);
    aggregateFavorites.setImage(PaintShop.loadImage("/img/icons/aggregate_all.gif"));
    aggregateFavorites.addDisposeListener(DisposeListenerImpl.getInstance());
    aggregateFavorites.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent e) {
        eventManager.actionAggregateAllCategories();
      }
    });

    /** Reload all Favorites */
    reloadFavorites = new MenuItem(favoritesMenu, SWT.PUSH);
    reloadFavorites.setImage(PaintShop.loadImage("/img/icons/reload_all.gif"));
    reloadFavorites.addDisposeListener(DisposeListenerImpl.getInstance());
    reloadFavorites.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent e) {
        eventManager.actionReloadAllCategories();
      }
    });

    /** Search all Favorites */
    searchFavorites = new MenuItem(favoritesMenu, SWT.PUSH);
    searchFavorites.setImage(PaintShop.loadImage("/img/icons/search_all.gif"));
    searchFavorites.addDisposeListener(DisposeListenerImpl.getInstance());
    searchFavorites.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent e) {
        eventManager.actionSearchInAllCategories();
      }
    });

    /** Separator */
    new MenuItem(favoritesMenu, SWT.SEPARATOR);

    /** Mark All Read */
    markAllRead = new MenuItem(favoritesMenu, SWT.PUSH);
    markAllRead.setImage(PaintShop.loadImage("/img/icons/mark_read.gif"));
    markAllRead.addDisposeListener(DisposeListenerImpl.getInstance());
    markAllRead.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent e) {
        eventManager.actionMarkAllCategoriesRead();
      }
    });

    /** Separator */
    new ToolItem(topToolBar, SWT.SEPARATOR);

    /** Search Newsfeeds */
    searchFeedItem = new ToolItem(topToolBar, SWT.PUSH);
    searchFeedItem.setImage(PaintShop.iconSearch);
    searchFeedItem.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent e) {
        eventManager.actionSearchFeeds();
      }
    });

    /** Discover Newsfeeds on Website */
    discoverFeedItem = new ToolItem(topToolBar, SWT.PUSH);
    discoverFeedItem.setImage(PaintShop.iconFeedDiscovery);
    discoverFeedItem.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent e) {
        eventManager.actionDiscoverFeeds();
      }
    });

    /** Validate Newsfeed */
    validateFeedItem = new ToolItem(topToolBar, SWT.PUSH);
    validateFeedItem.setImage(PaintShop.iconValidate);
    validateFeedItem.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent e) {
        eventManager.actionValidateFeeds(false);
      }
    });

    /** Pack the Top ToolBar */
    topToolBar.pack();
  }

  /**
   * Apply a wildcard popup menu to the text. Wildcards a displayed as "[TEXT]"
   * and represent replaceable parameters.
   * 
   * @param wildcards The wildcards to add to the menu
   */
  private void createWildCardMenu(String[] wildcards) {
    Menu wildCardMenu = new Menu(liveSearch);

    /** Foreach wildcards */
    for (int a = 0; a < wildcards.length; a++) {
      final String wildcard = wildcards[a];
      MenuItem menuItem = new MenuItem(wildCardMenu, SWT.POP_UP);
      menuItem.setText(wildcards[a]);
      if (!GlobalSettings.isMac())
        menuItem.setImage(PaintShop.iconBackward);
      menuItem.addSelectionListener(new SelectionAdapter() {
        public void widgetSelected(SelectionEvent e) {
          stopLiveSearch = true;
          liveSearch.insert(wildcard);
          stopLiveSearch = false;
        }
      });
    }
    liveSearch.setMenu(wildCardMenu);
  }

  /**
   * Get the index from the combo where the given URL is located or -1 if the
   * URL is not part of the combo
   * 
   * @param url Any URL to search for
   * @return int The index or -1
   */
  private int getIndex(String url) {
    for (int a = 0; a < urlTestCombo.getItemCount(); a++) {
      if (urlTestCombo.getItem(a).equals(url))
        return a;
    }
    return -1;
  }

  /** Init all components */
  private void initComponents() {

    /** ViewForm for the ToolBar controls */
    toolbarViewForm = new ViewForm(shell, SWT.BORDER | SWT.FLAT);
    toolbarViewForm.setLayoutData(new GridData(SWT.FILL, SWT.BEGINNING, true, false));
    toolbarViewForm.setBackground(PaintShop.grayViewFormColor);
    toolbarViewForm.setOnlyBorderTopBottom(true);

    /** Initialize the Context Menu */
    initContextMenu(toolbarViewForm);

    /** Top Container */
    topContainer = new Composite(toolbarViewForm, SWT.NONE);
    topContainer.setBackground(PaintShop.grayViewFormColor);
    topContainer.setLayout(LayoutShop.createFillLayout(0, 0));
    topContainer.setLayoutData(new GridData(SWT.FILL, SWT.BEGINNING, true, false));
    topContainer.setMenu(quickviewMenu);

    /** Create the Top ToolBar */
    createToolBar(topContainer);

    /** Apply to ViewForm */
    toolbarViewForm.setTopLeft(topContainer, false);

    /** Bottom Container GridLayout */
    GridLayout bottomContainerLayout = new GridLayout(5, true);
    bottomContainerLayout.marginWidth = 5;
    bottomContainerLayout.marginHeight = 3;
    bottomContainerLayout.marginTop = 1;

    /** Bottom Container */
    bottomContainer = new Composite(toolbarViewForm, SWT.NONE);
    bottomContainer.setLayout(bottomContainerLayout);
    bottomContainer.setLayoutData(new GridData(SWT.FILL, SWT.BEGINNING, true, false));
    bottomContainer.setBackground(PaintShop.grayViewFormColor);
    bottomContainer.setMenu(quickviewMenu);

    /** Apply to ViewForm */
    toolbarViewForm.setContent(bottomContainer, true);

    /** QuickView Container */
    Composite quickviewContainer = new Composite(bottomContainer, SWT.NONE);
    quickviewContainer.setLayout(LayoutShop.createGridLayout(3, 0, 0));
    quickviewContainer.setBackground(PaintShop.grayViewFormColor);
    quickviewContainer.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 4, 1));

    /** Label */
    urlLabelQuickView = new Label(quickviewContainer, SWT.LEFT);
    urlLabelQuickView.setBackground(PaintShop.grayViewFormColor);
    urlLabelQuickView.setLayoutData(new GridData(SWT.BEGINNING, SWT.CENTER, false, false));
    urlLabelQuickView.setText(GUI.i18n.getTranslation("LABEL_ADDRESS"));
    urlLabelQuickView.setFont(FontShop.dialogFont);
    urlLabelQuickView.setMenu(quickviewMenu);

    /** Combo containing the newsfeed's url */
    urlTestCombo = new Combo(quickviewContainer, SWT.DROP_DOWN);
    urlTestCombo.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
    urlTestCombo.setFont(FontShop.dialogFont);

    /** Allow to Drop Text into Combo */
    comboDNDTarget = new DropTarget(urlTestCombo, DND.DROP_MOVE);
    comboDNDTarget.setTransfer(new Transfer[] { TextTransfer.getInstance() });
    comboDNDTarget.addDropListener(new DropTargetAdapter() {
      public void drop(DropTargetEvent event) {

        /** Some Text is given */
        if (event.data != null && StringShop.isset(event.data.toString())) {

          /** First try to Extract Links */
          Vector links = new Vector();
          RegExShop.extractLinksFromText(event.data.toString(), links);
          if (links.size() > 0)
            urlTestCombo.setText(links.get(0).toString());

          /** No links given, just display the Text */
          else {
            String data = event.data.toString();

            /** This is a dragged TreeItem Category */
            if (data.indexOf(StringShop.CAT_TOKENIZER) >= 0) {
              String dataArray[] = data.split(StringShop.CAT_TOKENIZER);
              urlTestCombo.setText(dataArray[dataArray.length - 1]);
            }

            /** This is any other dragged Object containing Text */
            else {
              urlTestCombo.setText(org.jdom.Text.normalizeString(data));
            }
          }

          /** Transfer Focus into Combo */
          urlTestCombo.setFocus();
        }

        /** No valid Data given */
        else {
          event.feedback = DND.DROP_NONE;
        }
      }
    });

    /** Dispose DND Target on Combo Disposal */
    urlTestCombo.addDisposeListener(new DisposeListener() {
      public void widgetDisposed(DisposeEvent e) {
        comboDNDTarget.dispose();
      }
    });

    /** Open the selected URL from the combo */
    urlTestCombo.addSelectionListener(new SelectionAdapter() {

      /** Open the news feed that is entered in the Combo */
      public void widgetSelected(SelectionEvent e) {
        if (!StringShop.isWhiteSpaceOrEmpty(urlTestCombo.getText())) {
          eventManager.actionOpenFeed(urlTestCombo.getText().trim());
          shell.setFocus();
        }
      }
    });

    /** Open the URL when the user presses enter */
    urlTestCombo.addKeyListener(new KeyAdapter() {

      /** Open the news feed that is entered in the Combo */
      public void keyPressed(KeyEvent e) {
        if (e.character == SWT.CR && !StringShop.isWhiteSpaceOrEmpty(urlTestCombo.getText())) {
          addUrlToCombo(urlTestCombo.getText().trim());
          eventManager.actionOpenFeed(urlTestCombo.getText().trim());
          shell.setFocus();
        }
      }
    });

    /** Handle accelerators on focus gained / lost */
    urlTestCombo.addFocusListener(new FocusListener() {

      /** Disable printable accelerators while focus is in combo */
      public void focusGained(FocusEvent e) {
        rssOwlGui.getRSSOwlMenu().updateAccelerators(true);
      }

      /** Enable printable accelerators when focus is lost */
      public void focusLost(FocusEvent e) {
        rssOwlGui.getRSSOwlMenu().updateAccelerators();
      }
    });

    /** Live Search Container */
    Composite liveSearchContainer = new Composite(bottomContainer, SWT.NONE);
    liveSearchContainer.setLayout(LayoutShop.createGridLayout(3, 0, 0));
    liveSearchContainer.setBackground(PaintShop.grayViewFormColor);
    liveSearchContainer.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));

    /** Separator */
    Label sep = new Label(liveSearchContainer, SWT.SEPARATOR | SWT.VERTICAL);
    sep.setBackground(PaintShop.grayViewFormColor);
    GridData sepGridDataA = new GridData(GridData.HORIZONTAL_ALIGN_BEGINNING);
    sepGridDataA.heightHint = urlTestCombo.computeSize(SWT.DEFAULT, SWT.DEFAULT).y;
    sep.setLayoutData(sepGridDataA);
    sep.setMenu(quickviewMenu);

    /** Image Label */
    liveSearchLabel = new CLabel(liveSearchContainer, SWT.NONE);
    liveSearchLabel.setImage(PaintShop.iconFind);
    liveSearchLabel.setToolTipText(GUI.i18n.getTranslation("TOOLTIP_QUICKSEARCH"));
    liveSearchLabel.setBackground(PaintShop.grayViewFormColor);
    liveSearchLabel.setLayoutData(new GridData(SWT.BEGINNING, SWT.CENTER, false, false));
    liveSearchLabel.setMenu(quickviewMenu);

    /** Live Search Input Field */
    liveSearch = new Text(liveSearchContainer, SWT.BORDER);
    liveSearch.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
    liveSearch.setFont(FontShop.dialogFont);

    /** Perform Search upon Modify */
    liveSearch.addModifyListener(new ModifyListener() {
      public void modifyText(ModifyEvent e) {
        search();
      }
    });

    /** Search on Enter Key pressed */
    liveSearch.addKeyListener(new KeyAdapter() {
      public void keyPressed(KeyEvent e) {
        if (e.character == SWT.CR)
          rssOwlGui.getRSSOwlNewsTabFolder().searchInSelectedFeed(liveSearch.getText(), false);
      }
    });

    /** Handle accelerators on focus gained / lost */
    liveSearch.addFocusListener(new FocusListener() {

      /** Disable printable accelerators while focus is in combo */
      public void focusGained(FocusEvent e) {
        rssOwlGui.getRSSOwlMenu().updateAccelerators(true);
      }

      /** Enable printable accelerators when focus is lost */
      public void focusLost(FocusEvent e) {
        rssOwlGui.getRSSOwlMenu().updateAccelerators();
      }
    });

    /** Show wildcard menu for live search */
    createWildCardMenu(new String[] { " AND ", " OR ", " NOT " });

    /** Register to MenuManager */
    MenuManager.regiserLiveSearch(liveSearch);

    /** Setup Drop Support */
    WidgetShop.setupDropSupport(liveSearch, null);

    /** Tweak */
    WidgetShop.tweakTextWidget(liveSearch);

    /** Set Text */
    updateI18N();
  }

  /**
   * Initialize the Context Menu showing two items to hide/show ToolBar and
   * Quickview control.
   * 
   * @param parent The parent to use for Initialization.
   */
  private void initContextMenu(Control parent) {

    /** Context Menu */
    quickviewMenu = new Menu(parent);

    /** Set ToolBar Visbility */
    toolBarState = new MenuItem(quickviewMenu, SWT.CHECK);
    toolBarState.setSelection(GlobalSettings.isToolBarShown);
    toolBarState.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent e) {
        eventManager.actionShowToolBar(toolBarState.getSelection());
      }
    });

    /** Set Quickview Visibility */
    quickviewState = new MenuItem(quickviewMenu, SWT.CHECK);
    quickviewState.setSelection(GlobalSettings.isQuickviewShown);
    quickviewState.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent e) {
        eventManager.actionShowQuickview(quickviewState.getSelection());
      }
    });
  }

  /**
   * Add the URL to the combo if its not yet in
   * 
   * @param url Any URL
   */
  void addUrlToCombo(String url) {

    /** Only add URL to Combo if not yet existing */
    if (getIndex(url) != -1)
      return;

    /** Add if URL does not exist already */
    urlTestCombo.add(url);
  }

  /**
   * Perform the search in the displayed channel. Delay the search using a
   * Thread that first sleeps for a short amount of time. This allows the user
   * to complete the typing of a word before the search is beginning.
   */
  void search() {

    /** Check Flag */
    if (!stopLiveSearch) {

      /** Stop any running Search Delay Thread */
      if (searchDelayThread != null) {
        searchDelayThread.stopThread();
        searchDelayThread.interrupt();
      }

      /** Create a new Search Delay Thread */
      searchDelayThread = new ExtendedThread() {
        public void run() {

          /** Delay the Search */
          try {
            sleep(SEARCH_DELAY);
          }

          /** Search Term changed meanwhile, return */
          catch (InterruptedException e) {
            return;
          }

          /** Load Feed with Search pattern */
          if (!isStopped() && GUI.isAlive()) {
            GUI.display.syncExec(new Runnable() {
              public void run() {

                /** Reset Search Dialog */
                SearchDialog.lastSearch = liveSearch.getText();
                SearchDialog.wasSearchCaseSensitive = false;
                SearchDialog.wasSearchRegEx = false;
                SearchDialog.wasSearchWholeWord = false;

                /** Perform the Search */
                rssOwlGui.getRSSOwlNewsTabFolder().searchInSelectedFeed(liveSearch.getText(), false);
              }
            });
          }
        }
      };

      /** Start the Search Delay Thread */
      searchDelayThread.setName("Search Delay Thread");
      searchDelayThread.startThread();
    }
  }
}