/*   **********************************************************************  **
 **   Copyright notice                                                       **
 **                                                                          **
 **   (c) 2003-2005 RSSOwl Development Team                                  **
 **   http://www.rssowl.org/                                                 **
 **                                                                          **
 **   All rights reserved                                                    **
 **                                                                          **
 **   This program and the accompanying materials are made available under   **
 **   the terms of the Common Public License v1.0 which accompanies this     **
 **   distribution, and is available at:                                     **
 **   http://www.rssowl.org/legal/cpl-v10.html                               **
 **                                                                          **
 **   A copy is found in the file cpl-v10.html and important notices to the  **
 **   license from the team is found in the textfile LICENSE.txt distributed **
 **   in this package.                                                       **
 **                                                                          **
 **   This copyright notice MUST APPEAR in all copies of the file!           **
 **                                                                          **
 **   Contributors:                                                          **
 **     RSSOwl - initial API and implementation (bpasero@rssowl.org)         **
 **                                                                          **
 **  **********************************************************************  */

package net.sourceforge.rssowl.controller.dialog;

import net.sourceforge.rssowl.controller.EventManager;
import net.sourceforge.rssowl.controller.GUI;
import net.sourceforge.rssowl.controller.properties.AmphetaRateProperties;
import net.sourceforge.rssowl.controller.properties.BloggerProperties;
import net.sourceforge.rssowl.controller.properties.BrowserProperties;
import net.sourceforge.rssowl.controller.properties.ColorProperties;
import net.sourceforge.rssowl.controller.properties.ConnectionProperties;
import net.sourceforge.rssowl.controller.properties.FavoritesProperties;
import net.sourceforge.rssowl.controller.properties.FontProperties;
import net.sourceforge.rssowl.controller.properties.HotkeysProperties;
import net.sourceforge.rssowl.controller.properties.LanguageProperties;
import net.sourceforge.rssowl.controller.properties.MailProperties;
import net.sourceforge.rssowl.controller.properties.PropertyPage;
import net.sourceforge.rssowl.controller.properties.SortOrderPropertiers;
import net.sourceforge.rssowl.controller.properties.SystemTrayProperties;
import net.sourceforge.rssowl.controller.properties.ViewProperties;
import net.sourceforge.rssowl.controller.properties.WorkbenchProperties;
import net.sourceforge.rssowl.dao.SettingsSaver;
import net.sourceforge.rssowl.util.GlobalSettings;
import net.sourceforge.rssowl.util.i18n.ITranslatable;
import net.sourceforge.rssowl.util.shop.FontShop;
import net.sourceforge.rssowl.util.shop.LayoutDataShop;
import net.sourceforge.rssowl.util.shop.LayoutShop;
import net.sourceforge.rssowl.util.shop.PaintShop;
import net.sourceforge.rssowl.util.shop.WidgetShop;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeItem;

/**
 * The preferences dialog allows the user to change RSSOwl's preferences. The
 * user selects one of the categories from the left tree. Every category offers
 * a "Restore defaults" button to reset to the default settings. The "Apply" or
 * "OK" Button saves the made settings.
 * 
 * @author <a href="mailto:bpasero@rssowl.org">Benjamin Pasero </a>
 * @version 1.1.3
 */
public class PreferencesDialog extends Dialog implements ITranslatable {

  /** Remember the last opened property page */
  public static int lastOpenedPropertyPage = 0;

  /** Min. width of the dialog in DLUs */
  private static final int dialogMinWidth = 460;

  private TreeItem amphetaRate;
  private TreeItem blogger;
  private TreeItem browser;
  private Composite buttonHolder;
  private TreeItem colors;
  private TreeItem connection;
  private Composite contentHolder;
  private Button exportSettingsButton;
  private TreeItem favorites;
  private TreeItem font;
  private TreeItem formatNewsTipMail;
  private TreeItem hotkeys;
  private Composite importExportHolder;
  private Button importSettingsButton;
  private Label labelPrefTitle;
  private TreeItem language;
  private TreeItem sortOrder;
  private TreeItem systemTray;
  private String title;
  private TreeItem view;
  PropertyPage activePropertyPage;
  EventManager eventManager;
  Label labelImgHolder;
  Composite prefTitleHolder;
  GUI rssOwlGui;
  Tree tree;
  TreeItem workbench;

  /**
   * Creates a Dialog with a tree holding different categories. Each selected
   * tree item will open a view to change some settings. Note that the dialog
   * will have no visual representation (no widgets) until it is told to open.
   * <p>
   * Note that the <code>open</code> method blocks for input dialogs.
   * </p>
   * 
   * @param parentShell the parent shell
   * @param dialogTitle the dialog title, or <code>null</code> if none
   * @param rssOwlGui The MainController
   */
  public PreferencesDialog(Shell parentShell, String dialogTitle, GUI rssOwlGui) {
    super(parentShell);
    this.title = dialogTitle;
    this.rssOwlGui = rssOwlGui;
    this.eventManager = rssOwlGui.getEventManager();

    /** Init the PropertyChangeManager */
    PropertyPage.initPropertyChangeManager(rssOwlGui);
  }

  /** Update I18N - Is only called from the Language properties view! */
  public void updateI18N() {
    amphetaRate.setText("AmphetaRate");
    blogger.setText(GUI.i18n.getTranslation("MENU_BLOGGER"));
    colors.setText(GUI.i18n.getTranslation("MENU_COLORS"));
    workbench.setText(GUI.i18n.getTranslation("MENU_WORKBENCH"));
    view.setText(GUI.i18n.getTranslation("MENU_WINDOW"));
    browser.setText(GUI.i18n.getTranslation("MENU_BROWSER"));
    language.setText(GUI.i18n.getTranslation("MENU_LANGUAGE"));
    font.setText(GUI.i18n.getTranslation("MENU_FONT"));
    hotkeys.setText(GUI.i18n.getTranslation("MENU_HOTKEYS"));
    favorites.setText(GUI.i18n.getTranslation("HEADER_RSS_FAVORITES"));
    connection.setText(GUI.i18n.getTranslation("MENU_CONNECTION"));
    formatNewsTipMail.setText(GUI.i18n.getTranslation("MENU_NEWSTIP_MAIL"));
    sortOrder.setText(GUI.i18n.getTranslation("LABEL_SORT_ORDER"));
    getShell().setText(GUI.i18n.getTranslation("MENU_PREFERENCES"));

    if (WidgetShop.isset(systemTray))
      systemTray.setText(GUI.i18n.getTranslation("GROUP_TRAY"));

    /** Update the propertyview from which the method was called */
    createLanguageProps();

    /** Update Dialog Buttons */
    getButton(IDialogConstants.OK_ID).setText(GUI.i18n.getTranslation("BUTTON_OK"));
    getButton(IDialogConstants.CANCEL_ID).setText(GUI.i18n.getTranslation("BUTTON_CANCLE"));
    importSettingsButton.setText(GUI.i18n.getTranslation("MENU_IMPORT") + "...");
    exportSettingsButton.setText(GUI.i18n.getTranslation("MENU_EXPORT") + "...");

    /** Update Buttons Layout Data */
    setButtonLayoutData(importSettingsButton);
    setButtonLayoutData(exportSettingsButton);
    setButtonLayoutData(getButton(IDialogConstants.OK_ID));
    setButtonLayoutData(getButton(IDialogConstants.CANCEL_ID));
    importExportHolder.layout();
    buttonHolder.layout();

    /** Update layout */
    contentHolder.layout();
  }

  /** Create new PropertyPage: AmphetaRate */
  private void createAmphetaRateProps() {
    renewPropertyPage("AmphetaRate");
    activePropertyPage = new AmphetaRateProperties(contentHolder, rssOwlGui);
  }

  /** Create new PropertyPage: Blogger */
  private void createBloggerProps() {
    renewPropertyPage(GUI.i18n.getTranslation("MENU_BLOGGER"));
    activePropertyPage = new BloggerProperties(contentHolder);
  }

  /** Create new PropertyPage: Browser */
  private void createBrowserProps() {
    renewPropertyPage(GUI.i18n.getTranslation("MENU_BROWSER"));
    activePropertyPage = new BrowserProperties(contentHolder);
  }

  /** Create new PropertyPage: Colors */
  private void createColorProps() {
    renewPropertyPage(GUI.i18n.getTranslation("MENU_COLORS"));
    activePropertyPage = new ColorProperties(contentHolder, rssOwlGui);
  }

  /** Create new PropertyPage: Connection */
  private void createConnectionProps() {
    renewPropertyPage(GUI.i18n.getTranslation("MENU_CONNECTION"));
    activePropertyPage = new ConnectionProperties(contentHolder, rssOwlGui);
  }

  /** Create new PropertyPage: Favorites */
  private void createFavoritesProps() {
    renewPropertyPage(GUI.i18n.getTranslation("HEADER_RSS_FAVORITES"));
    activePropertyPage = new FavoritesProperties(contentHolder);
  }

  /** Create new PropertyPage: Font */
  private void createFontProps() {
    renewPropertyPage(GUI.i18n.getTranslation("MENU_FONT"));
    activePropertyPage = new FontProperties(contentHolder, rssOwlGui);
  }

  /** Create new PropertyPage: Hotkeys */
  private void createHotkeysProps() {
    renewPropertyPage(GUI.i18n.getTranslation("MENU_HOTKEYS"));
    activePropertyPage = new HotkeysProperties(contentHolder, rssOwlGui);
  }

  /** Create new PropertyPage: Language */
  private void createLanguageProps() {
    renewPropertyPage(GUI.i18n.getTranslation("MENU_LANGUAGE"));
    activePropertyPage = new LanguageProperties(contentHolder, rssOwlGui, this);
  }

  /** Create new PropertyPage: Mail */
  private void createMailProps() {
    renewPropertyPage(GUI.i18n.getTranslation("MENU_NEWSTIP_MAIL"));
    activePropertyPage = new MailProperties(contentHolder);
  }

  /** Create new PropertyPage: Sort Order */
  private void createSortOrderProps() {
    renewPropertyPage(GUI.i18n.getTranslation("LABEL_SORT_ORDER"));
    activePropertyPage = new SortOrderPropertiers(contentHolder);
  }

  /** Create new PropertyPage: System Tray */
  private void createSystemTrayProps() {
    renewPropertyPage(GUI.i18n.getTranslation("GROUP_TRAY"));
    activePropertyPage = new SystemTrayProperties(contentHolder, rssOwlGui);
  }

  /** Create new PropertyPage: View */
  private void createViewProps() {
    renewPropertyPage(GUI.i18n.getTranslation("MENU_WINDOW"));
    activePropertyPage = new ViewProperties(contentHolder, rssOwlGui);
  }

  /** Create new PropertyPage: Workbench */
  private void createWorkbenchProps() {
    renewPropertyPage(GUI.i18n.getTranslation("MENU_WORKBENCH"));
    activePropertyPage = new WorkbenchProperties(contentHolder, rssOwlGui);
  }

  /**
   * Returns true if the given Control c is a direct or indirect child of
   * container.
   * 
   * @param container The Container of the Control.
   * @param c The Control to check.
   * @return boolean TRUE if the Control is contained in the Container
   */
  private boolean isContained(Control container, Control c) {
    Composite parent;
    while ((parent = c.getParent()) != null) {
      if (parent == container)
        return true;
      c = parent;
    }
    return false;
  }

  /**
   * Renew the property page and set a new title
   * 
   * @param title A new title for the PropertyPage
   */
  private void renewPropertyPage(String title) {

    /** Update PropertyChangeManager and Dispose old PropertiesPage */
    if (activePropertyPage != null) {
      activePropertyPage.updatePropertiesChangeManager();
      activePropertyPage.dispose();
    }

    /** New preferences title */
    if (labelPrefTitle == null) {
      labelPrefTitle = new Label(prefTitleHolder, SWT.LEFT);
      labelPrefTitle.setBackground(GUI.display.getSystemColor(SWT.COLOR_WHITE));
      labelPrefTitle.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
      labelPrefTitle.setFont(FontShop.dialogBoldFont);
    }

    /** Image for the title */
    if (labelImgHolder == null) {
      labelImgHolder = new Label(prefTitleHolder, SWT.NONE);
      labelImgHolder.setImage(PaintShop.iconBlueStripes);
      labelImgHolder.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_END));
    }

    /** Update */
    labelPrefTitle.setText(title);
    labelPrefTitle.update();
    labelImgHolder.update();
    prefTitleHolder.layout();
    contentHolder.layout();
  }

  /**
   * Set the selected tree item
   * 
   * @param propertyPage The propertypage number
   */
  private void setTreeSelection(int propertyPage) {
    switch (propertyPage) {

      /** Workbench */
      case 0:
        tree.setSelection(new TreeItem[] { workbench });
        break;

      /** Colors */
      case 1:
        tree.setSelection(new TreeItem[] { colors });
        break;

      /** Fonts */
      case 2:
        tree.setSelection(new TreeItem[] { font });
        break;

      /** Hotkeys */
      case 3:
        tree.setSelection(new TreeItem[] { hotkeys });
        break;

      /** Language */
      case 4:
        tree.setSelection(new TreeItem[] { language });
        break;

      /** Sort Order */
      case 5:
        tree.setSelection(new TreeItem[] { sortOrder });
        break;

      /** View */
      case 6:
        tree.setSelection(new TreeItem[] { view });
        break;

      /** System Tray */
      case 7:
        tree.setSelection(new TreeItem[] { systemTray });
        break;

      /** AmphetaRate */
      case 8:
        tree.setSelection(new TreeItem[] { amphetaRate });
        break;

      /** Blogger */
      case 9:
        tree.setSelection(new TreeItem[] { blogger });
        break;

      /** Browser */
      case 10:
        tree.setSelection(new TreeItem[] { browser });
        break;

      /** Favorites */
      case 11:
        tree.setSelection(new TreeItem[] { favorites });
        break;

      /** Format NewsTip Mail */
      case 12:
        tree.setSelection(new TreeItem[] { formatNewsTipMail });
        break;

      /** Connection */
      case 13:
        tree.setSelection(new TreeItem[] { connection });
        break;
    }
  }

  /**
   * @see org.eclipse.jface.dialogs.Dialog#buttonPressed(int)
   */
  protected void buttonPressed(int buttonId) {

    /** Apply settings, save settings */
    if (buttonId == IDialogConstants.OK_ID)
      saveSettings();

    /** Cleanup */
    if (activePropertyPage != null)
      activePropertyPage.dispose();

    super.buttonPressed(buttonId);
  }

  /**
   * @see org.eclipse.jface.window.Window#configureShell(org.eclipse.swt.widgets.Shell)
   */
  protected void configureShell(Shell shell) {
    shell.setLayout(LayoutShop.createGridLayout(1, 0, 5));

    /** On Mac do not set Shell Image since it will change the Dock Image */
    if (!GlobalSettings.isMac())
      shell.setImages(PaintShop.iconOwl);

    shell.setText(title);
    shell.setSize(0, 0);

    /** Listen for the shell to dispose */
    shell.addDisposeListener(new DisposeListener() {
      public void widgetDisposed(DisposeEvent e) {

        /** Reset a flag */
        FontProperties.setPossibleFontChange(false);
      }
    });
  }

  /**
   * @see org.eclipse.jface.dialogs.Dialog#createButtonBar(org.eclipse.swt.widgets.Composite)
   */
  protected Control createButtonBar(Composite parent) {

    /** Composite to hold the bottom buttons */
    buttonHolder = new Composite(parent, SWT.NONE);
    buttonHolder.setLayout(LayoutShop.createGridLayout(2, 0, 0, 5, 5, false));
    buttonHolder.setLayoutData(LayoutDataShop.createGridData(GridData.FILL_HORIZONTAL, 2));

    /** Composite to store Import / Export buttons */
    importExportHolder = new Composite(buttonHolder, SWT.NONE);
    importExportHolder.setLayout(LayoutShop.createGridLayout(2, 5, 5, 5));
    importExportHolder.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

    /** Import GlobalSettings from file */
    importSettingsButton = new Button(importExportHolder, SWT.NONE);
    importSettingsButton.setText(GUI.i18n.getTranslation("MENU_IMPORT") + "...");
    importSettingsButton.setFont(FontShop.dialogFont);
    setButtonLayoutData(importSettingsButton);
    importSettingsButton.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent e) {
        boolean success = eventManager.actionImportSettings(getShell());

        /** Exit settings dialog upon success */
        if (success)
          buttonPressed(IDialogConstants.CANCEL_ID);
      }
    });

    /** Export GlobalSettings into file */
    exportSettingsButton = new Button(importExportHolder, SWT.NONE);
    exportSettingsButton.setFont(FontShop.dialogFont);
    exportSettingsButton.setText(GUI.i18n.getTranslation("MENU_EXPORT") + "...");
    setButtonLayoutData(exportSettingsButton);
    exportSettingsButton.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent e) {

        /** First save Settings */
        saveSettings();

        /** Export */
        eventManager.actionExportSettings(getShell());
      }
    });

    /** Composite to store Ok / Cancel buttons */
    Composite okCancelHolder = new Composite(buttonHolder, SWT.NONE);
    okCancelHolder.setLayout(LayoutShop.createGridLayout(2, 0, 5, 5));
    okCancelHolder.setLayoutData(new GridData(SWT.END, SWT.TOP, false, false));

    /** Button order on Mac is different */
    if (GlobalSettings.isMac()) {

      /** Create OK / Cancel Buttons */
      createButton(okCancelHolder, IDialogConstants.CANCEL_ID, GUI.i18n.getTranslation("BUTTON_CANCLE"), false).setFont(FontShop.dialogFont);
      createButton(okCancelHolder, IDialogConstants.OK_ID, GUI.i18n.getTranslation("BUTTON_OK"), true).setFont(FontShop.dialogFont);
    } else {

      /** Create OK / Cancel Buttons */
      createButton(okCancelHolder, IDialogConstants.OK_ID, GUI.i18n.getTranslation("BUTTON_OK"), true).setFont(FontShop.dialogFont);
      createButton(okCancelHolder, IDialogConstants.CANCEL_ID, GUI.i18n.getTranslation("BUTTON_CANCLE"), false).setFont(FontShop.dialogFont);
    }

    return buttonHolder;
  }

  /**
   * @see org.eclipse.jface.dialogs.Dialog#createDialogArea(org.eclipse.swt.widgets.Composite)
   */
  protected Control createDialogArea(Composite parent) {

    /** Composite to hold all components */
    Composite baseComposite = (Composite) super.createDialogArea(parent);
    baseComposite.setLayout(LayoutShop.createGridLayout(2, 0, 5, 15, 0, false));
    baseComposite.setLayoutData(LayoutDataShop.createGridData(GridData.FILL_BOTH, 1));

    /** Composite to hold the tree */
    Composite treeHolder = new Composite(baseComposite, SWT.NONE);
    treeHolder.setLayoutData(LayoutDataShop.createGridData(GridData.FILL_VERTICAL, 1, convertHorizontalDLUsToPixels(140)));
    treeHolder.setLayout(LayoutShop.createGridLayout(1, 5, 0));

    /** Tree holding all categorys for settings */
    tree = new Tree(treeHolder, SWT.BORDER);
    tree.setFont(FontShop.dialogFont);
    tree.setLayoutData(new GridData(GridData.FILL_BOTH));
    tree.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent e) {
        handleTreeItemSelect();
      }
    });

    /** Expand / Collapse item on pressed enter key */
    tree.addKeyListener(new KeyAdapter() {
      public void keyPressed(KeyEvent e) {
        TreeItem selectedItem = tree.getSelection()[0];
        if (e.keyCode == SWT.CR && selectedItem.getItemCount() > 0)
          selectedItem.setExpanded(!selectedItem.getExpanded());
      }
    });

    /** Expand / Collapse selection on double click */
    tree.addListener(SWT.MouseDoubleClick, new Listener() {
      public void handleEvent(Event event) {
        if (tree.getSelectionCount() > 0) {
          Rectangle clickedRect = event.getBounds();
          Rectangle selectedRect = tree.getSelection()[0].getBounds();

          /** Only handle event, if Mouse is over treeitem */
          if (selectedRect.contains(clickedRect.x, clickedRect.y))
            tree.getSelection()[0].setExpanded(!tree.getSelection()[0].getExpanded());
        }
      }
    });

    /** Category: Workbench */
    workbench = new TreeItem(tree, SWT.NONE);
    workbench.setText(GUI.i18n.getTranslation("MENU_WORKBENCH"));

    /** Category: Colors */
    colors = new TreeItem(workbench, SWT.NONE);
    colors.setText(GUI.i18n.getTranslation("MENU_COLORS"));

    /** Category: Font */
    font = new TreeItem(workbench, SWT.NONE);
    font.setText(GUI.i18n.getTranslation("MENU_FONT"));

    /** Category: Hotkeys */
    hotkeys = new TreeItem(workbench, SWT.NONE);
    hotkeys.setText(GUI.i18n.getTranslation("MENU_HOTKEYS"));

    /** Category: Language */
    language = new TreeItem(workbench, SWT.NONE);
    language.setText(GUI.i18n.getTranslation("MENU_LANGUAGE"));

    /** Category: Sort Order */
    sortOrder = new TreeItem(workbench, SWT.NONE);
    sortOrder.setText(GUI.i18n.getTranslation("LABEL_SORT_ORDER"));

    /** Category: View */
    view = new TreeItem(workbench, SWT.NONE);
    view.setText(GUI.i18n.getTranslation("MENU_WINDOW"));

    /** Category: System Tray (if supported) */
    if (GlobalSettings.useSystemTray()) {
      systemTray = new TreeItem(view, SWT.NONE);
      systemTray.setText(GUI.i18n.getTranslation("GROUP_TRAY"));
    }

    /** Category: AmphetaRate */
    amphetaRate = new TreeItem(tree, SWT.NONE);
    amphetaRate.setText("AmphetaRate");

    /** Category: Blogger */
    blogger = new TreeItem(tree, SWT.NONE);
    blogger.setText(GUI.i18n.getTranslation("MENU_BLOGGER"));

    /** Category: Browser */
    browser = new TreeItem(tree, SWT.NONE);
    browser.setText(GUI.i18n.getTranslation("MENU_BROWSER"));

    /** Category: Favorites */
    favorites = new TreeItem(tree, SWT.NONE);
    favorites.setText(GUI.i18n.getTranslation("HEADER_RSS_FAVORITES"));

    /** Category: Format NewsTip mail */
    formatNewsTipMail = new TreeItem(tree, SWT.NONE);
    formatNewsTipMail.setText(GUI.i18n.getTranslation("MENU_NEWSTIP_MAIL"));

    /** Category: Connection */
    connection = new TreeItem(tree, SWT.NONE);
    connection.setText(GUI.i18n.getTranslation("MENU_CONNECTION"));

    /** Composite holding the content page to change settings */
    contentHolder = new Composite(baseComposite, SWT.NONE);
    contentHolder.setLayoutData(new GridData(GridData.FILL_BOTH | GridData.VERTICAL_ALIGN_BEGINNING));
    contentHolder.setLayout(LayoutShop.createGridLayout(1, 5, 0));

    /** Composite holding the title of the preference page */
    prefTitleHolder = new Composite(contentHolder, SWT.NONE);
    prefTitleHolder.setLayout(LayoutShop.createGridLayout(2));
    prefTitleHolder.setLayoutData(LayoutDataShop.createGridData(GridData.FILL_HORIZONTAL, 2));
    prefTitleHolder.setBackground(GUI.display.getSystemColor(SWT.COLOR_WHITE));

    /** Change painting of the composite holding the title */
    prefTitleHolder.addPaintListener(new PaintListener() {
      public void paintControl(PaintEvent e) {
        e.gc.setForeground(GUI.display.getSystemColor(SWT.COLOR_DARK_GRAY));
        Rectangle bounds = prefTitleHolder.getClientArea();
        bounds.height -= 2;
        bounds.width -= 1;
        e.gc.drawRectangle(bounds);
      }
    });

    /** Select an element in the tree */
    setTreeSelection(lastOpenedPropertyPage);
    handleTreeItemSelect();

    /** Seperate from OK and Cancel Button */
    Label seperator = new Label(baseComposite, SWT.HORIZONTAL | SWT.SEPARATOR);
    seperator.setLayoutData(LayoutDataShop.createGridData(GridData.HORIZONTAL_ALIGN_BEGINNING | GridData.FILL_HORIZONTAL, 2));

    /** Explicitly show Workbench expanded */
    workbench.setExpanded(true);

    /** Explicitly show View expanded */
    if (GlobalSettings.useSystemTray())
      view.setExpanded(true);

    /** If shown, scroll the Bar back to start */
    if (WidgetShop.isset(tree.getHorizontalBar()))
      tree.getHorizontalBar().setSelection(0);

    return contentHolder;
  }

  /**
   * @see org.eclipse.jface.dialogs.Dialog#initializeBounds()
   */
  protected void initializeBounds() {
    initializeBounds(true);
  }

  /**
   * Set size and location of the dialog
   * 
   * @param updateLocation If TRUE also update the location of the dialog
   */
  protected void initializeBounds(boolean updateLocation) {

    /** On Mac OS X the default button must be the right-most button */
    if (GlobalSettings.isMac()) {
      Shell shell = getShell();
      if (shell != null) {
        Button defaultButton = shell.getDefaultButton();
        if (defaultButton != null && isContained(buttonBar, defaultButton))
          defaultButton.moveBelow(null);
      }
    }

    /** Define Shell Bounds */
    Point currentSize = getShell().getSize();
    Point bestSize = getShell().computeSize(convertHorizontalDLUsToPixels(dialogMinWidth), SWT.DEFAULT);

    /** Calculate new location or use old one */
    Point location = (updateLocation == true) ? getInitialLocation(bestSize) : getShell().getLocation();

    /** Only change if dialog needs to become bigger */
    if (updateLocation && bestSize.y > currentSize.y)
      getShell().setBounds(location.x, location.y, bestSize.x, bestSize.y);
    else if (bestSize.y > currentSize.y)
      getShell().setSize(bestSize.x, bestSize.y);

    /** Also define minimium size */
    getShell().setMinimumSize(bestSize.x, bestSize.y);
  }

  /**
   * Set the layout data of the button to a GridData with appropriate widths
   * This method was slightly modified so that it is not setting a heightHint.
   * 
   * @param button The button to layout
   */
  protected void setButtonLayoutData(Button button) {
    GridData data = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
    int widthHint = convertHorizontalDLUsToPixels(IDialogConstants.BUTTON_WIDTH);
    data.widthHint = Math.max(widthHint, button.computeSize(SWT.DEFAULT, SWT.DEFAULT, true).x);
    button.setLayoutData(data);
  }

  /**
   * @see org.eclipse.jface.window.Window#setShellStyle(int)
   */
  protected void setShellStyle(int newShellStyle) {
    super.setShellStyle(SWT.CLOSE | SWT.TITLE | SWT.RESIZE | SWT.APPLICATION_MODAL);
  }

  /** Handle selection of items in the tree and create new PropertyPage */
  void handleTreeItemSelect() {

    /** Return if no item is selected */
    if (tree.getSelection().length == 0)
      return;

    /** Workbench */
    if (tree.getSelection()[0].getText().equals(GUI.i18n.getTranslation("MENU_WORKBENCH"))) {
      createWorkbenchProps();
      lastOpenedPropertyPage = 0;
    }

    /** Colors */
    else if (tree.getSelection()[0].getText().equals(GUI.i18n.getTranslation("MENU_COLORS"))) {
      createColorProps();
      lastOpenedPropertyPage = 1;
    }

    /** Fonts */
    else if (tree.getSelection()[0].getText().equals(GUI.i18n.getTranslation("MENU_FONT"))) {
      createFontProps();
      lastOpenedPropertyPage = 2;
    }

    /** Hotkeys */
    else if (tree.getSelection()[0].getText().equals(GUI.i18n.getTranslation("MENU_HOTKEYS"))) {
      createHotkeysProps();
      lastOpenedPropertyPage = 3;
    }

    /** Language */
    else if (tree.getSelection()[0].getText().equals(GUI.i18n.getTranslation("MENU_LANGUAGE"))) {
      createLanguageProps();
      lastOpenedPropertyPage = 4;
    }

    /** Sort Order */
    else if (tree.getSelection()[0].getText().equals(GUI.i18n.getTranslation("LABEL_SORT_ORDER"))) {
      createSortOrderProps();
      lastOpenedPropertyPage = 5;
    }

    /** View */
    else if (tree.getSelection()[0].getText().equals(GUI.i18n.getTranslation("MENU_WINDOW"))) {
      createViewProps();
      lastOpenedPropertyPage = 6;
    }

    /** System Tray */
    else if (tree.getSelection()[0].getText().equals(GUI.i18n.getTranslation("GROUP_TRAY"))) {
      createSystemTrayProps();
      lastOpenedPropertyPage = 7;
    }

    /** AmphetaRate */
    else if (tree.getSelection()[0].getText().equals("AmphetaRate")) {
      createAmphetaRateProps();
      lastOpenedPropertyPage = 8;
    }

    /** Blogger */
    else if (tree.getSelection()[0].getText().equals(GUI.i18n.getTranslation("MENU_BLOGGER"))) {
      createBloggerProps();
      lastOpenedPropertyPage = 9;
    }

    /** Browser */
    else if (tree.getSelection()[0].getText().equals(GUI.i18n.getTranslation("MENU_BROWSER"))) {
      createBrowserProps();
      lastOpenedPropertyPage = 10;
    }

    /** Favorites */
    else if (tree.getSelection()[0].getText().equals(GUI.i18n.getTranslation("HEADER_RSS_FAVORITES"))) {
      createFavoritesProps();
      lastOpenedPropertyPage = 11;
    }

    /** Format NewsTip Mail */
    else if (tree.getSelection()[0].getText().equals(GUI.i18n.getTranslation("MENU_NEWSTIP_MAIL"))) {
      createMailProps();
      lastOpenedPropertyPage = 12;
    }

    /** Connection */
    else if (tree.getSelection()[0].getText().equals(GUI.i18n.getTranslation("MENU_CONNECTION"))) {
      createConnectionProps();
      lastOpenedPropertyPage = 13;
    }

    /** Update layout */
    contentHolder.layout();

    /** Update shell size */
    initializeBounds(false);
  }

  /**
   * Update all Settings and Save them.
   */
  void saveSettings() {

    /** First apply active Page */
    activePropertyPage.applyButtonPressed();

    /** Save Properties */
    PropertyPage.getPropertyChangeManager().saveProperties();

    /** Save all settings to the user.xml file */
    new SettingsSaver(rssOwlGui).saveUserSettings();
  }
}