/*   **********************************************************************  **
 **   Copyright notice                                                       **
 **                                                                          **
 **   (c) 2003-2005 RSSOwl Development Team                                  **
 **   http://www.rssowl.org/                                                 **
 **                                                                          **
 **   All rights reserved                                                    **
 **                                                                          **
 **   This program and the accompanying materials are made available under   **
 **   the terms of the Common Public License v1.0 which accompanies this     **
 **   distribution, and is available at:                                     **
 **   http://www.rssowl.org/legal/cpl-v10.html                               **
 **                                                                          **
 **   A copy is found in the file cpl-v10.html and important notices to the  **
 **   license from the team is found in the textfile LICENSE.txt distributed **
 **   in this package.                                                       **
 **                                                                          **
 **   This copyright notice MUST APPEAR in all copies of the file!           **
 **                                                                          **
 **   Contributors:                                                          **
 **     RSSOwl - initial API and implementation (bpasero@rssowl.org)         **
 **                                                                          **
 **  **********************************************************************  */

package net.sourceforge.rssowl.controller.popup;

import net.sourceforge.rssowl.controller.DisposeListenerImpl;
import net.sourceforge.rssowl.controller.EventManager;
import net.sourceforge.rssowl.controller.GUI;
import net.sourceforge.rssowl.model.Category;
import net.sourceforge.rssowl.util.GlobalSettings;
import net.sourceforge.rssowl.util.shop.PaintShop;
import net.sourceforge.rssowl.util.shop.ProxyShop;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MenuAdapter;
import org.eclipse.swt.events.MenuEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.TreeItem;

/**
 * The blogroll-category popup appears when right-clicking on any category that
 * is part of a blogroll in the tree.
 * 
 * @author <a href="mailto:bpasero@rssowl.org">Benjamin Pasero </a>
 * @version 1.1.3
 */
public class BlogrollCategoryPopup {
  private Menu popUpMenu;
  private Shell shell;
  EventManager eventManager;
  GUI rssOwlGui;
  TreeItem selectedTreeItem;

  /**
   * Instantiate a new BlogrollCategoryPopup
   * 
   * @param shell The shell
   * @param rssOwlGui The maincontroller
   * @param popUpMenu The menu to add this menu to
   * @param eventManager The event manager
   * @param selectedTreeItem The treeitem that gets this menu
   */
  public BlogrollCategoryPopup(Shell shell, GUI rssOwlGui, Menu popUpMenu, EventManager eventManager, TreeItem selectedTreeItem) {
    this.shell = shell;
    this.rssOwlGui = rssOwlGui;
    this.popUpMenu = popUpMenu;
    this.eventManager = eventManager;
    this.selectedTreeItem = selectedTreeItem;

    /** Init the Popup */
    initBlogrollCategoryMenu();
  }

  /** Init the popup menuStructure for a blogroll category */
  private void initBlogrollCategoryMenu() {

    /** Aggregate selected category */
    MenuItem open = new MenuItem(popUpMenu, SWT.CASCADE);
    open.setText(GUI.i18n.getTranslation("POP_AGGREGATE_FAV"));
    open.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent e) {
        eventManager.actionAggregateCategory(false);
      }
    });

    /** Disable this menuitem if treeitem has no childs */
    if (selectedTreeItem.getItemCount() <= 0)
      open.setEnabled(false);

    /** Separator */
    new MenuItem(popUpMenu, SWT.SEPARATOR);

    /** Mark selected category read */
    MenuItem markRead = new MenuItem(popUpMenu, SWT.CASCADE);
    markRead.setText(GUI.i18n.getTranslation("POP_MARK_CATEGORY_READ"));
    markRead.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent e) {
        eventManager.actionMarkCategoryRead();
      }
    });

    /** Disable this menuitem if treeitem has no childs */
    if (selectedTreeItem.getItemCount() <= 0)
      markRead.setEnabled(false);

    /** Separator */
    new MenuItem(popUpMenu, SWT.SEPARATOR);

    /** Reload all favorites of the selected category */
    MenuItem reload = new MenuItem(popUpMenu, SWT.CASCADE);
    reload.setText(GUI.i18n.getTranslation("BUTTON_RELOAD_CAT"));
    if (!GlobalSettings.isMac())
      reload.setImage(PaintShop.iconReload);
    reload.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent e) {
        eventManager.actionAggregateCategory(true);
      }
    });

    /** Disable this menuitem if treeitem has no childs */
    if (selectedTreeItem.getItemCount() <= 0)
      reload.setEnabled(false);

    /** Separator */
    new MenuItem(popUpMenu, SWT.SEPARATOR);

    /** Search in all favorites of the selected category */
    MenuItem search = new MenuItem(popUpMenu, SWT.CASCADE);
    search.setText(GUI.i18n.getTranslation("SEARCH_DIALOG_TITLE") + "...");
    if (!GlobalSettings.isMac()) {
      search.setImage(PaintShop.loadImage("/img/icons/search.gif"));
      search.addDisposeListener(DisposeListenerImpl.getInstance());
    }
    search.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent e) {
        eventManager.actionSearchInCategory();
      }
    });

    /** Disable this menuitem if treeitem has no childs */
    if (selectedTreeItem.getItemCount() <= 0)
      search.setEnabled(false);

    /** Separator */
    new MenuItem(popUpMenu, SWT.SEPARATOR);

    /** New Menu */
    MenuItem newCatOrFav = new MenuItem(popUpMenu, SWT.CASCADE);
    newCatOrFav.setText(GUI.i18n.getTranslation("POP_NEW"));

    Menu selectNewCatOrFav = new Menu(shell, SWT.DROP_DOWN);
    newCatOrFav.setMenu(selectNewCatOrFav);
    if (!GlobalSettings.isMac()) {
      newCatOrFav.setImage(PaintShop.loadImage("/img/icons/add_category_small.gif"));
      newCatOrFav.addDisposeListener(DisposeListenerImpl.getInstance());
    }

    /** Create a new category */
    MenuItem cascadeItemNewCat = new MenuItem(selectNewCatOrFav, SWT.CASCADE);
    cascadeItemNewCat.setText(GUI.i18n.getTranslation("LABEL_CATEGORY") + "...");
    if (!GlobalSettings.isMac()) {
      cascadeItemNewCat.setImage(PaintShop.loadImage("/img/icons/add_category_pop.gif"));
      cascadeItemNewCat.addDisposeListener(DisposeListenerImpl.getInstance());
    }
    cascadeItemNewCat.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent e) {
        eventManager.actionNewCategory(false);
      }
    });

    /** Synchronize */
    MenuItem synchronize = new MenuItem(popUpMenu, SWT.CASCADE);
    synchronize.setText(GUI.i18n.getTranslation("POP_SYNCHRONIZE"));
    synchronize.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent e) {
        eventManager.actionSynchronizeBlogroll();
      }
    });

    /** Unsubscribe */
    MenuItem remove = new MenuItem(popUpMenu, SWT.CASCADE);
    remove.setText(GUI.i18n.getTranslation("POP_UNSUBSCRIBE"));
    if (!GlobalSettings.isMac()) {
      remove.setImage(PaintShop.loadImage("/img/icons/delete.gif"));
      remove.addDisposeListener(DisposeListenerImpl.getInstance());
    }
    remove.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent e) {
        eventManager.actionDeleteBlogroll();
      }
    });

    /** Separator */
    new MenuItem(popUpMenu, SWT.SEPARATOR);

    /** Copy URL of the blogroll */
    MenuItem copyXmlLocation = new MenuItem(popUpMenu, SWT.CASCADE);
    copyXmlLocation.setText(GUI.i18n.getTranslation("POP_COPY_NEWS_URL"));
    copyXmlLocation.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent e) {
        eventManager.actionCopyBlogrollXmlLocation();
      }
    });

    /** Separator */
    new MenuItem(popUpMenu, SWT.SEPARATOR);

    /** Import OPML files */
    MenuItem importMenu = new MenuItem(popUpMenu, SWT.CASCADE);
    importMenu.setText(GUI.i18n.getTranslation("POP_IMPORT"));
    if (!GlobalSettings.isMac()) {
      importMenu.setImage(PaintShop.loadImage("/img/icons/import.gif"));
      importMenu.addDisposeListener(DisposeListenerImpl.getInstance());
    }

    Menu selectImport = new Menu(shell, SWT.DROP_DOWN);
    importMenu.setMenu(selectImport);

    /** Import OPML */
    MenuItem cascadeItemImportOPML = new MenuItem(selectImport, SWT.CASCADE);
    cascadeItemImportOPML.setText(GUI.i18n.getTranslation("POP_FROM_OPML") + "...");
    if (!GlobalSettings.isMac()) {
      cascadeItemImportOPML.setImage(PaintShop.loadImage("/img/icons/import_opml.gif"));
      cascadeItemImportOPML.addDisposeListener(DisposeListenerImpl.getInstance());
    }
    cascadeItemImportOPML.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent e) {
        eventManager.actionImportOPML();
      }
    });

    /** Import Synchronized Blogroll */
    MenuItem cascadeItemImportBlogroll = new MenuItem(selectImport, SWT.CASCADE);
    cascadeItemImportBlogroll.setText(GUI.i18n.getTranslation("POP_IMPORT_BLOGROLL") + "...");
    if (!GlobalSettings.isMac()) {
      cascadeItemImportBlogroll.setImage(PaintShop.loadImage("/img/icons/add_subscription.gif"));
      cascadeItemImportBlogroll.addDisposeListener(DisposeListenerImpl.getInstance());
    }
    cascadeItemImportBlogroll.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent e) {
        eventManager.actionImportBlogroll();
      }
    });

    /** Export to OPML file */
    MenuItem export = new MenuItem(popUpMenu, SWT.CASCADE);
    export.setText(GUI.i18n.getTranslation("BUTTON_EXPORT"));
    if (!GlobalSettings.isMac()) {
      export.setImage(PaintShop.loadImage("/img/icons/export.gif"));
      export.addDisposeListener(DisposeListenerImpl.getInstance());
    }

    /** Disable this menuitem if treeitem has no childs */
    if (selectedTreeItem.getItemCount() <= 0)
      export.setEnabled(false);

    Menu selectexport = new Menu(shell, SWT.DROP_DOWN);
    export.setMenu(selectexport);

    MenuItem cascadeItemCreateOPML = new MenuItem(selectexport, SWT.CASCADE);
    cascadeItemCreateOPML.setText(GUI.i18n.getTranslation("POP_EXPORT_OPML") + "...");
    if (!GlobalSettings.isMac()) {
      cascadeItemCreateOPML.setImage(PaintShop.loadImage("/img/icons/export_opml.gif"));
      cascadeItemCreateOPML.addDisposeListener(DisposeListenerImpl.getInstance());
    }
    cascadeItemCreateOPML.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent e) {
        eventManager.actionCategoryToOPML();
      }
    });

    /** In case Proxy is activated */
    if (ProxyShop.isUseProxy()) {

      /** Separator */
      new MenuItem(popUpMenu, SWT.SEPARATOR);

      /** Enable / Disable use of proxy */
      MenuItem proxy = new MenuItem(popUpMenu, SWT.CASCADE);
      proxy.setText(GUI.i18n.getTranslation("MENU_PROXY"));

      Menu selectProxy = new Menu(shell, SWT.DROP_DOWN);
      proxy.setMenu(selectProxy);

      final MenuItem cascadeItemProxy = new MenuItem(selectProxy, SWT.CASCADE | SWT.CHECK);
      cascadeItemProxy.setText(GUI.i18n.getTranslation("POP_USEPROXY"));
      cascadeItemProxy.addSelectionListener(new SelectionAdapter() {
        public void widgetSelected(SelectionEvent e) {
          if (cascadeItemProxy.isEnabled())
            eventManager.actionSetProxyOnTreeItem(cascadeItemProxy.getSelection());
        }
      });

      /** Dynamically react on the Proxy Settings */
      selectProxy.addMenuListener(new MenuAdapter() {
        public void menuShown(MenuEvent e) {
          Category selectedCat = rssOwlGui.getRSSOwlFavoritesTree().getSelectedCat();

          if (selectedCat != null)
            cascadeItemProxy.setSelection(selectedCat.isUseProxy());
        }
      });
    }

    /** Separator */
    new MenuItem(popUpMenu, SWT.SEPARATOR);

    /** Edit selected blogroll */
    MenuItem properties = new MenuItem(popUpMenu, SWT.CASCADE);
    properties.setText(GUI.i18n.getTranslation("POP_PROPERTIES"));
    properties.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent e) {
        eventManager.actionEditBlogroll();
      }
    });
  }
}